/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.configurator;

import java.util.Properties;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Hashtable;
import java.util.Vector;

import java.net.URI;
import java.net.URISyntaxException;

import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.Connection;
import javax.xml.registry.RegistryService;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.BulkResponse;
import javax.xml.registry.JAXRException;

import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.Association;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.RegistryObject;
import javax.xml.registry.infomodel.SpecificationLink;
import javax.xml.registry.infomodel.Classification;
import javax.xml.registry.infomodel.ServiceBinding;

import com.sun.wsi.scm.util.WSIConstants;
import com.sun.wsi.scm.util.JAXRConstants;

import com.sun.wsi.scm.configuration.ConfigurationEndpointType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;

import javax.xml.rpc.server.ServiceLifecycle;
import javax.xml.rpc.server.ServletEndpointContext;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;

import org.xml.sax.SAXException;

import java.io.InputStream;
import java.io.IOException;

public class ConfiguratorPortTypeImpl implements ConfiguratorPortType, ServiceLifecycle, WSIConstants, JAXRConstants {
	Hashtable tModelKeys = new Hashtable();
	Hashtable rolesCategoryBag = new Hashtable();

	BusinessQueryManager bqm = null;
	BusinessLifeCycleManager lcm = null;

	Log logger = null;
	ServletContext servletContext = null;

	public void init(Object context) {
		logger = LogFactory.getLog(LOGGER);

		tModelKeys.put(LOGGING_TMODEL_KEY, "Logging");
		tModelKeys.put(RETAILER_TMODEL_KEY, "Retailer");
		tModelKeys.put(WAREHOUSE_TMODEL_KEY, "Warehouse");
		tModelKeys.put(MANUFACTURER_TMODEL_KEY, "Manufacturer");
		//tModelKeys.put(CONFIGURATOR_TMODEL_KEY, "Configurator");
		rolesCategoryBag.put(WAREHOUSE_KEYVALUE[WAREHOUSE_A], ConfigurationEndpointRole.WarehouseA);
		rolesCategoryBag.put(WAREHOUSE_KEYVALUE[WAREHOUSE_B], ConfigurationEndpointRole.WarehouseB);
		rolesCategoryBag.put(WAREHOUSE_KEYVALUE[WAREHOUSE_C], ConfigurationEndpointRole.WarehouseC);
		rolesCategoryBag.put(MANUFACTURER_KEYVALUE[MANUFACTURER_A], ConfigurationEndpointRole.ManufacturerA);
		rolesCategoryBag.put(MANUFACTURER_KEYVALUE[MANUFACTURER_B], ConfigurationEndpointRole.ManufacturerB);
		rolesCategoryBag.put(MANUFACTURER_KEYVALUE[MANUFACTURER_C], ConfigurationEndpointRole.ManufacturerC);

		ServletEndpointContext servletEndpointContext = (ServletEndpointContext)context;
		servletContext = servletEndpointContext.getServletContext();
	}

	public ConfigOptionsType getConfigurationOptions(boolean refresh)
		throws ConfiguratorFailedFault {
		ConfigOptionsType configOptionsType = new ConfigOptionsType();

		ConfigOptionType[] configOptions = null;

		// Cached data is stored in /WEB-INF/endpoints.xml.
		// Currently, this is a snapshot of endpoints and is
		// not in sync with a fresh pull from UDDI registry
		if (refresh)
			configOptions = getConfigOptionsFromUDDI();
		else {
			InputStream is = servletContext.getResourceAsStream("/WEB-INF/endpoints.xml");
			configOptions = getConfigOptionsFromCache(is);
		}

		configOptionsType.setConfigOption(configOptions);

		return configOptionsType;
	}


	private ConfigOptionType[] getConfigOptionsFromUDDI() throws ConfiguratorFailedFault {
		ConfigOptionType[] configOptions = null;
		Properties props = new Properties();
		//props.setProperty(QUERY_MANAGER_URL, System.getProperty("query.manager"));
		//props.setProperty(LIFECYCLE_MANAGER_URL, System.getProperty("lifecycle.manager"));
		props.setProperty(QUERY_MANAGER_URL, "http://uddi.ibm.com/ubr/inquiryapi");
		props.setProperty(LIFECYCLE_MANAGER_URL, "https://uddi.ibm.com/ubr/publishapi");

		try {
			ConnectionFactory connectionFactory = ConnectionFactory.newInstance();
			connectionFactory.setProperties(props);
			Connection connection = connectionFactory.createConnection();

			RegistryService registryService = connection.getRegistryService();
			bqm = registryService.getBusinessQueryManager();
			lcm = registryService.getBusinessLifeCycleManager();

			// Retrieve the set of uddi:businessKeys for businesses 
			// that have reciprocal relationships with the WS-I
			// uddi:businessEntity
			ArrayList orgList = showcaseEntities();

			// Retrieve each uddi:businessEntity element to get to the
			// uddi:businessService categorization and name, and the
			// uddi:bindingTemplate accessPoint and instanceParams
			// elements.
			configOptions = showcaseServices((Organization[])(orgList.toArray(new Organization[0])));

		} catch (Throwable t) {
			logger.error(t.getMessage(), t);
			throw new ConfiguratorFailedFault(t.getMessage());
		}

		return configOptions;
	}

	/**
	* Retrieve the set of uddi:businessKeys for businesses 
	* that have reciprocal relationships with the WS-I
	* uddi:businessEntity
	*/
	private ArrayList showcaseEntities() throws JAXRException {
		logger.info("Retrieving uddi:businessKeys with reciprocal relationship");

		ArrayList orgList = new ArrayList();
		Key wsiKey = lcm.createKey(WSI_BUSINESS_KEY);

		BulkResponse response = bqm.findAssociations(null, WSI_BUSINESS_KEY, null, null);
		Iterator iter = response.getCollection().iterator();
		while (iter.hasNext()) {
			Association association = (Association)iter.next();
			Organization sourceOrg = (Organization)association.getSourceObject();
			Organization targetOrg = (Organization)association.getTargetObject();
			if (logger.isDebugEnabled()) {
				logger.debug("\"" + sourceOrg.getName().getValue() + " relatedTo \"" + targetOrg.getName().getValue() + "\"");
				logger.debug("confirmed: " + association.isConfirmed());
			}
			if (association.isConfirmed()) {
				logger.info(targetOrg.getName().getValue() + " has confirmed relationship with WS-I businessEntity");
				orgList.add(targetOrg);
			} else {
				logger.info(targetOrg.getName().getValue() + " does not have confirmed relationship with WS-I businessEntity");
			}
		}

		return orgList;
	}

	/**
	* Retrieve each uddi:businessEntity element to get to the
	* uddi:businessService categorization and name, and the
	* uddi:bindingTemplate accessPoint and instanceParams
	* elements
	*/
	private ConfigOptionType[] showcaseServices(Organization[] orgs) throws JAXRException, URISyntaxException{
		logger.info("Retrieve services for the showcase business entities");
		Vector configVector = new Vector();
		logger.info(orgs.length + " # of orgs");

		for (int i=0; i<orgs.length; i++) {
			String orgName = orgs[i].getName().getValue();
			logger.info("Getting services for \"" + orgName + "\" organization ...");
			Collection findQualifiers = new ArrayList();
			findQualifiers.add(FindQualifier.OR_ALL_KEYS);
			Collection namePatterns = new ArrayList();
			namePatterns.add("%");
			BulkResponse response = bqm.findServices(orgs[i].getKey(), findQualifiers, namePatterns, null, null);
			Collection services = response.getCollection();
			Iterator serviceIter = services.iterator();

			int serviceCount = 0;
			while (serviceIter.hasNext()) {
				logger.info("");
				Service service = (Service)serviceIter.next();
				String serviceName = service.getName().getValue();
				logger.info("\t" + ++serviceCount + "th service: " + serviceName);

				Iterator bindingsIter = service.getServiceBindings().iterator();
				while (bindingsIter.hasNext()) {
					ServiceBinding bindingTemplate = (ServiceBinding)bindingsIter.next();
					String accessPoint = bindingTemplate.getAccessURI();
					if (logger.isDebugEnabled())
						logger.debug("\taccessPoint: " + accessPoint);
					Collection specLinks = bindingTemplate.getSpecificationLinks();
					Iterator iter = specLinks.iterator();
					while (iter.hasNext()) {
						SpecificationLink specLink = (SpecificationLink)iter.next();
						Concept tModel = (Concept)specLink.getSpecificationObject();
						if (logger.isDebugEnabled())
							logger.debug(tModel.getName().getValue() + "{" + tModel.getKey().getId() + "} tModel found");
						Iterator tModelInstanceInfoIter = specLink.getUsageParameters().iterator();

						// ignore the configurator service
						if (tModel.getKey().getId().equals(CONFIGURATOR_TMODEL_KEY)) {
							logger.info("Ignoring the Configurator service");
							continue;
						}

						// if a tmodel is not a showcase model, then it
						// should not be included
						if (!isShowcaseTmodel(tModel)) {
							if (logger.isDebugEnabled())
								logger.debug(tModel.getName().getValue() + "{" + tModel.getKey().getId() + "} is not a showcase tModel.");
							continue;
						}
						
						// prepare the selection params
						String instanceParams = "";
						while (tModelInstanceInfoIter.hasNext()) {
							String instanceParam = (String)tModelInstanceInfoIter.next();
							instanceParams += instanceParam;
						}
						if (logger.isDebugEnabled())
							logger.debug("\tinstanceParams: " + instanceParams);


						// set the appropriate role
						ConfigurationEndpointRole role = null;
						Iterator categoryBag = service.getClassifications().iterator();
						if (logger.isDebugEnabled())
							logger.debug("Searching for categorizations ...");
						if (!categoryBag.hasNext()) {
							if (logger.isDebugEnabled())
								logger.debug("\tcategorization: empty");
						} else {
							while (categoryBag.hasNext()) {
								Classification classifi = (Classification)categoryBag.next();
								String categoryBagKey = classifi.getClassificationScheme().getKey().getId();
								if (logger.isDebugEnabled())
									logger.debug("\tcategorization: {" + classifi.getName().getValue() + "}" + classifi.getValue());
								if (categoryBagKey.equals(CATEGORY_BAG_KEY)) {
									role = (ConfigurationEndpointRole)rolesCategoryBag.get(classifi.getName().getValue() + "," + classifi.getValue());
								}
							}
						}
						if (logger.isDebugEnabled())
							logger.debug("... search complete.");

						// role==null - true for configurator, loggingFacility
						// and retailer
						// configurator service is ignored earlier itself
						// set the roles for retailer and loggingFacility
						if (role == null) {
							if (tModel.getKey().getId().equals(RETAILER_TMODEL_KEY)) {
								if (logger.isDebugEnabled())
									logger.debug("Setting role for Retailer ...");
								role = ConfigurationEndpointRole.Retailer;
							} else if (tModel.getKey().getId().equals(LOGGING_TMODEL_KEY)) {
								if (logger.isDebugEnabled())
									logger.debug("Setting role for LoggingFacility ...");
								role = ConfigurationEndpointRole.LoggingFacility;
							} else {
								if (logger.isDebugEnabled())
									logger.debug("No role set for " + tModel.getName().getValue() + "{" + tModel.getKey().getId() + "}");
							}
						}
						logger.info("\trole: " + ((role == null) ? "null" : role.getValue()));
						ConfigOptionType configOptionType = prepareConfigOption(serviceName, instanceParams, accessPoint, role);
						configVector.add(configOptionType);
					}
				}
			}
		}

		return (ConfigOptionType[])configVector.toArray(new ConfigOptionType[0]);
	}

	boolean isShowcaseTmodel(Concept tModel) throws JAXRException {
		return tModelKeys.containsKey(tModel.getKey().getId());
	}

	ConfigOptionType prepareConfigOption(String name, String instanceParams, String accessPoint, ConfigurationEndpointRole role) throws JAXRException, URISyntaxException {
		ConfigOptionType configOptionType = new ConfigOptionType();

		configOptionType.setName(name);
		configOptionType.setSelectionParms(instanceParams);

		ConfigurationEndpointType configEndpoint = new ConfigurationEndpointType();
		configEndpoint.set_value(new URI(accessPoint));
		if (role != null)
			configEndpoint.setRole(role);
		configOptionType.setConfigurationEndpoint(configEndpoint);

		return configOptionType;
	}

	ConfigOptionType[] getConfigOptionsFromCache(InputStream is) {
		ConfigOptionType[] configOption = null;
		logger.info("Reading the endpoints from local cache");
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document = db.parse(is);

			NodeList nodelist = document.getElementsByTagName("service");
			configOption = new ConfigOptionType[nodelist.getLength()];
			for (int i=0; i<nodelist.getLength(); i++) {
				logger.info((i+1) + "th service");
				Element serviceElem = (Element)nodelist.item(i);
				configOption[i] = new ConfigOptionType();

				// service name
				NamedNodeMap nnm = serviceElem.getParentNode().getAttributes();
				String orgName = nnm.getNamedItem("name").getNodeValue();
				nnm = serviceElem.getAttributes();
				String serviceType = nnm.getNamedItem("type").getNodeValue();
				if (logger.isDebugEnabled())
					logger.debug("name: " + orgName + " " + serviceType);
				configOption[i].setName(orgName + " " + serviceType);

				// endpoint and selection-params
				String endpoint = "";
				String selectionParams = "";
				NodeList serviceChildren = serviceElem.getChildNodes();
				for (int j=0; j<serviceChildren.getLength(); j++) {
					if (serviceChildren.item(j).getNodeType() == Node.ELEMENT_NODE) {
						// can contain only two element child
						NodeList nl = serviceChildren.item(j).getChildNodes();
						Node node = nl.item(0);
						// one of them being "endpoint"
						if (serviceChildren.item(j).getNodeName().equals("endpoint")) {
							endpoint = node.getNodeValue();
						} else { // and other being "selection-params"
							selectionParams = (node != null) ?  node.getNodeValue() : "";
						}
					}
				}

				if (logger.isDebugEnabled()) {
					logger.debug("selection params: " + selectionParams);
					logger.debug("endpoint: " + endpoint);
				}

				// selection params
				configOption[i].setSelectionParms(selectionParams);

				ConfigurationEndpointType configEndpointType = new ConfigurationEndpointType();
				// configuration endpoint - URI
				configEndpointType.set_value(new URI(endpoint));

				// configuration endpoint - role
				ConfigurationEndpointRole role = null;
				if (serviceType.equals(SERVICE_NAMES[LOGGING_SERVICE]))
					role = ConfigurationEndpointRole.LoggingFacility;	
				else if (serviceType.equals(SERVICE_NAMES[RETAILER_SERVICE]))
					role = ConfigurationEndpointRole.Retailer;
				else if (serviceType.equals(SERVICE_NAMES[WAREHOUSEA_SERVICE]))
					role = ConfigurationEndpointRole.WarehouseA;
				else if (serviceType.equals(SERVICE_NAMES[WAREHOUSEB_SERVICE]))
					role = ConfigurationEndpointRole.WarehouseB;
				else if (serviceType.equals(SERVICE_NAMES[WAREHOUSEC_SERVICE]))
					role = ConfigurationEndpointRole.WarehouseC;
				else if (serviceType.equals(SERVICE_NAMES[MANUFACTURERA_SERVICE]))
					role = ConfigurationEndpointRole.ManufacturerA;
				else if (serviceType.equals(SERVICE_NAMES[MANUFACTURERB_SERVICE]))
					role = ConfigurationEndpointRole.ManufacturerB;
				else if (serviceType.equals(SERVICE_NAMES[MANUFACTURERC_SERVICE]))
					role = ConfigurationEndpointRole.ManufacturerC;
				if (logger.isDebugEnabled()) {
					logger.debug("service type: " + serviceType);
					logger.debug("role: " + role);
				}
				configEndpointType.setRole(role);

				configOption[i].setConfigurationEndpoint(configEndpointType);
				logger.info("");
			}
		} catch (ParserConfigurationException ex) {
			logger.error(ex);
		} catch (URISyntaxException ex) {
			logger.error(ex);
		} catch (SAXException ex) {
			logger.error(ex);
		} catch (IOException ex) {
			logger.error(ex);
		}

		return configOption;
	}

	public void destroy() { }

}


