/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.manufacturer;

import java.util.Hashtable;

import java.rmi.RemoteException;

import javax.xml.rpc.Stub;
import javax.xml.rpc.server.ServiceLifecycle;
import com.sun.xml.rpc.client.StubBase;

import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationFaultType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;

import com.sun.wsi.scm.manufacturer.po.PurchOrdType;
import com.sun.wsi.scm.manufacturer.po.Item;
import com.sun.wsi.scm.manufacturer.po.ItemList;
import com.sun.wsi.scm.manufacturer.po.SubmitPOFaultType_Exception;

import com.sun.wsi.scm.manufacturer.cb.StartHeaderType;

import com.sun.wsi.scm.logging.LogEventRequestType;
import com.sun.wsi.scm.logging.LoggingFacilityService_Impl;
import com.sun.wsi.scm.logging.LoggingFacilityLogPortType;

import com.sun.wsi.scm.util.ConfigurationValidator;
import com.sun.wsi.scm.util.WSIConstants;

import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ManufacturerBPortTypeImpl implements ManufacturerPortType, WSIConstants, ServiceLifecycle {

	LoggingFacilityLogPortType logStub = null;
	LogEventRequestType logRequest = new LogEventRequestType();

	Log logger = null;

	public void init(Object context) {
		logger = LogFactory.getLog(LOGGER);

		LoggingFacilityService_Impl logService = new LoggingFacilityService_Impl();
		logStub = logService.getLoggingFacilityPort();
		if (logger.isTraceEnabled())
			((StubBase)logStub)._setTransportFactory(new HttpClientTransportFactory(System.out));
	}

	public boolean submitPO(PurchOrdType purchaseOrder, ConfigurationType configurationHeader, StartHeaderType startHeader) throws ConfigurationFaultType, SubmitPOFaultType_Exception, RemoteException {

		if (logger.isTraceEnabled())
			logger.trace(MANUFACTURERB_SUBMIT_PO);
		// ** - Validate the configurationHeader
		Hashtable roles = null;
		roles = ConfigurationValidator.validateHeader(configurationHeader);

		// Set the endpoint address of the logging facility
		// from the configurationHeader
		((Stub)logStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.LoggingFacility));

		logRequest.setDemoUserID(configurationHeader.getUserId());
		logRequest.setServiceID(MANUFACTURERB_SUBMIT_PO);
		logRequest.setEventID("UC3-3");
		ItemList itemList = purchaseOrder.getItems();
		Item[] items = itemList.getItem();

		String desc = "";
		for (int i=0; i<items.length; i++) {
			desc += items[i].getID();
			desc += (i == items.length-1) ? "" : ", ";
		}
		logRequest.setEventDescription("ManufacturerB is replenishing stock for " + desc);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());
		logStub.logEvent(logRequest);

		// Validate the PO
		POValidator.validateOrder(purchaseOrder.getItems().getItem(), MANUFACTURERB_PRODUCTS, MANUFACTURERB_DATA);

		HeaderValidator.validateStartHeader(startHeader, ConfigurationEndpointRole.ManufacturerB);

		// Spawn a new thread for callback invocation
		if (logger.isTraceEnabled())
			logger.trace("Invoking the callback from " + MANUFACTURERA);
		CallbackInvoker callback = new CallbackInvoker(purchaseOrder, configurationHeader, startHeader);
		Thread thread = new Thread(callback);
		thread.start();

		Hashtable qtyHash = new Hashtable();
		for (int i=0; i<MANUFACTURERB_DATA.length; i++)
			qtyHash.put(new Integer(MANUFACTURERB_PRODUCTS[i]), new Integer(MANUFACTURERB_DATA[i][STOCK]));

		for (int i=0; i<items.length; i++) {
			desc = "ManufacturerB";
			Integer stockQty = (Integer)qtyHash.get(new Integer(items[i].getID().intValue()));
			
			if (items[i].getQty() > stockQty.intValue()) {
				logRequest.setEventID("UC5-5");
				desc += " has produced additional units of " + items[i].getID() + " and is shipping " + items[i].getQty() + " units";
			} else {
				logRequest.setEventID("UC4-1");
				desc += " is shipping " + items[i].getID() + " from existing inventory";
			}

			logRequest.setEventDescription(desc);
			if (logger.isDebugEnabled())
				logger.debug(logRequest.getEventDescription());
			logStub.logEvent(logRequest);
		}


		// If everything looks good, return a correct response
		return true;
	}

	public void destroy() { }
}


