/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.retailer;

import com.sun.wsi.scm.retailer.RetailerService_Impl;
import com.sun.wsi.scm.retailer.RetailerPortType;

import com.sun.wsi.scm.retailer.CatalogType;
import com.sun.wsi.scm.retailer.CatalogItem;

import com.sun.wsi.scm.retailer.CustomerDetailsType;
import com.sun.wsi.scm.retailer.PartsOrderType;
import com.sun.wsi.scm.retailer.PartsOrderResponseType;
import com.sun.wsi.scm.retailer.PartsOrderItem;

import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;

import com.sun.wsi.scm.logging.LogEntry;
import com.sun.wsi.scm.logging.LoggingFacilityService_Impl;
import com.sun.wsi.scm.logging.LoggingFacilityLogPortType;
import com.sun.wsi.scm.logging.GetEventsFaultType;
import com.sun.wsi.scm.logging.GetEventsRequestType;
import com.sun.wsi.scm.logging.GetEventsResponseType;

import java.math.BigInteger;
import java.math.BigDecimal;

import javax.xml.rpc.Stub;
import com.sun.xml.rpc.client.http.HttpClientTransportFactory;
import com.sun.xml.rpc.client.StubBase;

import java.io.File;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;

import java.text.SimpleDateFormat;

import java.rmi.RemoteException;

import java.util.Date;
import java.util.Properties;
import java.util.Vector;
import java.util.StringTokenizer;

import com.sun.wsi.scm.util.XMLWriter;
import com.sun.wsi.scm.util.AttributeNV;
import com.sun.wsi.scm.util.WSIConstants;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.net.URI;

public class RetailerClient implements WSIConstants {
	RetailerPortType retailerStub = null;
	LoggingFacilityLogPortType loggingStub = null;
	SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss yyyy.MM.dd");
	static String[] demoUserId = new String[6];

	PrintStream xmlLog = null;

	static Log logger = null;

	public static void main(String[] args) {
		LogFactory logFactory = LogFactory.getFactory();
		logger = logFactory.getInstance(LOGGER);
		logger.info("Starting Retailer client ...");

		try {
			// process all the configurations
			RetailerClient client = new RetailerClient();
			XMLWriter.writeStartTag("all-orders");

			Properties props = new Properties();
			// Read the endpoint properties file
			File file = new File(System.getProperty("endpoints.props"));	
			FileInputStream is = new FileInputStream(file);
			props.load(is);

			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			String configFileName = System.getProperty("order.config.file");
			Document document = db.parse(new File(configFileName));
			NodeList nodelist = document.getElementsByTagName("configuration");

			file = new File(System.getProperty("log.home") + System.getProperty("file.separator") + System.getProperty("retailer.soap.msgs.file"));
			OutputStream retailerLog = new FileOutputStream(file);

			file = new File(System.getProperty("log.home") + System.getProperty("file.separator") + System.getProperty("logging.soap.msgs.file"));
			OutputStream loggingLog = new FileOutputStream(file);

			for (int i=0; i<nodelist.getLength(); i++) {
				Element elem = (Element)nodelist.item(i);

				NamedNodeMap nnm = elem.getAttributes();
				Node attNameNode = nnm.getNamedItem("name");
				String vendor = attNameNode.getNodeValue();

				Node attEnabledNode = nnm.getNamedItem("enabled");
				String enabled = attEnabledNode.getNodeValue();

				if (!enabled.equals("true"))
					continue;

				// process each configuration
				NodeList nl = elem.getElementsByTagName("serviceURL");

				// process all the serviceURLs
				ConfigurationType configHeader = new ConfigurationType();
				configHeader.setMustUnderstand(false);

				ConfigurationEndpointType[] configEndpoint = new ConfigurationEndpointType[8];

				URI loggingURI = null;
				URI retailerURI = null;
				for (int j=0; j<nl.getLength(); j++) {
					Node serviceURLNode = nl.item(j);

					// process each serviceURL
					nnm = serviceURLNode.getAttributes();
					attNameNode = nnm.getNamedItem("name");
					String serviceName = attNameNode.getNodeValue();
					String id = serviceURLNode.getFirstChild().getNodeValue();
					if (!props.containsKey(id))
						throw new IllegalArgumentException(id + " not found");

					URI endpointURI = new URI(props.getProperty(id));
					if (logger.isDebugEnabled()) {
						logger.debug("serviceName: " + serviceName);
						logger.debug("endpoint: " + endpointURI);
					}
					if (serviceName.equals("LoggingFacility"))
						loggingURI = endpointURI;
					else if (serviceName.equals("Retailer"))
						retailerURI = endpointURI;


					// populate configuration header
					configEndpoint[j] = new ConfigurationEndpointType();
					configEndpoint[j].setRole(ConfigurationEndpointRole.fromValue(serviceName));
					configEndpoint[j].set_value(endpointURI);

				}
				configHeader.setServiceUrl(configEndpoint);

				// set the logging and retailer endpoints
				client.setEndpoints(retailerURI.toString(), loggingURI.toString(), retailerLog, loggingLog);

				logger.info("Invoking \"" + vendor + "\" endpoints ...");
				// invoke the orders
				XMLWriter.writeStartTag("orders");
				client.logConfigHeader(configHeader, vendor);
				demoUserId = client.getDemoUserId(vendor);


				if (configFileName.indexOf("demo") != -1) {
					configHeader.setUserId(demoUserId[0]);
					client.demoScript(configHeader);
				} else {
					// Identify the order that needs to be executed
					if (logger.isDebugEnabled())
						logger.debug("Identifying the order that needs to be executed");
					String orders = System.getProperty("orders");
					Vector orderVector = new Vector();
					if (orders == null)
						throw new IllegalArgumentException("No \"orders\" specified. Please specify a system property \"orders\" with value \"all\" or any combination of 1 - 7 separated through commas");

					if (logger.isDebugEnabled())
						logger.debug("orders: " + orders);
					if (orders.equals("all")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding all the orders");
						orderVector.add("1");
						orderVector.add("2");
						orderVector.add("3");
						orderVector.add("4");
						orderVector.add("5");
						orderVector.add("6");
						orderVector.add("7");
					} else {
						StringTokenizer tokens = new StringTokenizer(orders, ",");
						while (tokens.hasMoreElements()) {
							orderVector.add(tokens.nextToken());	
						}
					}

					if (orderVector.contains("1")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 1");
						configHeader.setUserId(demoUserId[0]);
						client.order1(configHeader, vendor);
					}

					if (orderVector.contains("2")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 2");
						configHeader.setUserId(demoUserId[1]);
						client.order2(configHeader, vendor);
					}

					if (orderVector.contains("3")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 3");
						configHeader.setUserId(demoUserId[2]);
						client.order3(configHeader, vendor);
					}

					if (orderVector.contains("4")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 4");
						configHeader.setUserId(demoUserId[3]);
						client.order4(configHeader, vendor);
					}

					if (orderVector.contains("5")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 5");
						configHeader.setUserId(demoUserId[4]);
						client.order5(configHeader, vendor);
					}

					if (orderVector.contains("6")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 6");
						configHeader.setUserId(demoUserId[5]);
						client.order6(configHeader, vendor);
					}

					if (orderVector.contains("7")) {
						if (logger.isDebugEnabled())
							logger.debug("Adding order 7");
						client.getCatalog(vendor);	
					}
				}

				XMLWriter.writeEndTag("orders");
				logger.info("");
			}
			logger.info("Retailer client placed the orders successfully and generated the logs in " + System.getProperty("log.home") + System.getProperty("file.separator") + System.getProperty("log.file"));
		} catch (Throwable ex) {
			ex.printStackTrace(System.out);	
		} finally {
			XMLWriter.writeEndTag("all-orders");
		}
	}

	private void setEndpoints(String retailerURI, String loggingURI, OutputStream retailerLog, OutputStream loggingLog) {
		RetailerService_Impl retailerService = new RetailerService_Impl();
		retailerStub = retailerService.getRetailerPort();
		((Stub)retailerStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, retailerURI);
		if (logger.isTraceEnabled())
			((StubBase)retailerStub)._setTransportFactory(new HttpClientTransportFactory(retailerLog));

		LoggingFacilityService_Impl loggingService = new LoggingFacilityService_Impl();
		loggingStub = loggingService.getLoggingFacilityPort();
		((Stub)loggingStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, loggingURI);
		if (logger.isTraceEnabled())
			((StubBase)loggingStub)._setTransportFactory(new HttpClientTransportFactory(loggingLog));
	}

	private String[] getDemoUserId(String vendor) {
		long now = (new Date()).getTime();

		String[] demoUserId = new String[6];

		for (int i=0; i<6; i++) {
			demoUserId[i] = "order" + String.valueOf((i+1)) + "." + vendor + "." + String.valueOf(now+i);
		}

		return demoUserId;
	}

	/**
	* Prepares the log files and initializes the XMLWriter
	*/
	public RetailerClient() throws IOException {
		File file = new File(System.getProperty("log.home") + System.getProperty("file.separator") + System.getProperty("log.file"));
		file.delete();
		xmlLog = new PrintStream(new FileOutputStream(file));
		XMLWriter.setWriter(xmlLog);
	}

	/**
	* Logs the configuration header (only the serviceURLs)
	*/
	private void logConfigHeader(ConfigurationType configHeader, String vendor) {
		XMLWriter.writeStartTag("configuration", new AttributeNV("vendor", vendor));
		ConfigurationEndpointType[] configEndpoint = configHeader.getServiceUrl();
		for (int i=0; i<configEndpoint.length; i++) {
			XMLWriter.writeContent("serviceURL", new AttributeNV("name", configEndpoint[i].getRole().getValue()), configEndpoint[i].get_value().toString());
		}
		XMLWriter.writeEndTag("configuration");
	}


	private CustomerDetailsType getCustomerDetails() {
		CustomerDetailsType customerDetails = new CustomerDetailsType();

		if (logger.isDebugEnabled())
			logger.debug("Setting the customer details");
		customerDetails.setCustnbr("A55555-99999BB-abc");
		customerDetails.setName("Sun Microsystems, Inc.");
		customerDetails.setStreet1("4140, Network Circle");
		customerDetails.setStreet2("MS: SCA14-304");
		customerDetails.setCity("Santa Clara");
		customerDetails.setState("CA");
		customerDetails.setZip("95054");
		customerDetails.setCountry("USA");

		return customerDetails;
	}


	private void logResponse(PartsOrderResponseType response, String demoUserId) {

		// only if response != null (e.g. order4 passes null here)
		if (response != null) {

			PartsOrderResponseItem[] responseItem = response.getItem();
			XMLWriter.writeStartTag("response");
			if (logger.isDebugEnabled())
				logger.debug("Response entries ...");
			try {
				for (int i=0; i<responseItem.length; i++) {
					if (logger.isDebugEnabled()) {
						logger.debug((i+1) + "th product");
						logger.debug("Product Number : " + responseItem[i].getProductNumber());
						logger.debug("Quantity       : " + responseItem[i].getQuantity());
						logger.debug("Price          : " + responseItem[i].getPrice().setScale(2, BigDecimal.ROUND_HALF_UP));
						logger.debug("Comment        : " + responseItem[i].getComment());
						logger.debug("");
					}

					XMLWriter.writeStartTag("item");
					XMLWriter.writeContent("product", responseItem[i].getProductNumber().toString());
					XMLWriter.writeContent("quantity", responseItem[i].getQuantity().toString());
					XMLWriter.writeContent("price", responseItem[i].getPrice().setScale(2, BigDecimal.ROUND_HALF_UP).toString());
					XMLWriter.writeContent("comment", responseItem[i].getComment());
					XMLWriter.writeEndTag("item");
				}
			} catch (Throwable ex) {
				ex.printStackTrace(System.out);
			} finally {
				XMLWriter.writeEndTag("response");
			}
		}

		if (logger.isDebugEnabled())
			logger.debug("Logging entries ...");
		try {
			XMLWriter.writeStartTag("logs");
			GetEventsRequestType request = new GetEventsRequestType();
			request.setDemoUserID(demoUserId);

			// Wait for double the callback delay
			// Just to make sure that all the entries are logged
			Thread.currentThread().sleep(CALLBACK_DELAY + CALLBACK_DELAY);
			GetEventsResponseType logResponse = loggingStub.getEvents(request);
			LogEntry[] logEntry = logResponse.getLogEntry();
			for (int i=0; i<logEntry.length; i++) {
				String date = sdf.format(logEntry[i].getTimestamp().getTime());
				if (logger.isDebugEnabled()) {
					logger.debug((i+1) + "th log entry");
					logger.debug(date);
					logger.debug(logEntry[i].getServiceID());
					logger.debug(logEntry[i].getEventID());
					logger.debug(logEntry[i].getEventDescription());
					logger.debug("");
				}

				XMLWriter.writeStartTag("log");
				XMLWriter.writeContent("timestamp", date);
				XMLWriter.writeContent("event-id", logEntry[i].getEventID());
				XMLWriter.writeContent("service-id", logEntry[i].getServiceID());
				XMLWriter.writeContent("description", logEntry[i].getEventDescription());
				XMLWriter.writeEndTag("log");
			}
		} catch (GetEventsFaultType ex) {
			ex.printStackTrace(System.out);
		} catch (RemoteException ex) {
			ex.printStackTrace(System.out);
		} catch (Throwable ex) {
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("logs");
		}
	}

	private void getCatalog(String vendor) {
		logger.info("Invoking getCatalog \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "7"));
		try {
			CatalogType catalog = retailerStub.getCatalog();
			XMLWriter.writeStartTag("catalog");

			CatalogItem[] catalogItems = catalog.getItem();
			logger.debug("getCatalog response entries ...");
			for (int i=0; i<catalogItems.length; i++) {
				if (logger.isDebugEnabled()) {
					logger.debug("-->" + (i+1) + "th product");
					logger.debug(catalogItems[i].getProductNumber());
					logger.debug(catalogItems[i].getName());
					logger.debug(catalogItems[i].getDescription());
					logger.debug(catalogItems[i].getCategory());
					logger.debug(catalogItems[i].getBrand());
					logger.debug(catalogItems[i].getPrice());
					logger.debug("");
				}

				XMLWriter.writeStartTag("catalog-item");
				XMLWriter.writeContent("product", catalogItems[i].getProductNumber().toString());
				XMLWriter.writeContent("name", catalogItems[i].getName());
				XMLWriter.writeContent("description", catalogItems[i].getDescription());
				XMLWriter.writeContent("category", catalogItems[i].getCategory());
				XMLWriter.writeContent("brand", catalogItems[i].getBrand());
				XMLWriter.writeContent("price", catalogItems[i].getPrice().toString());
				XMLWriter.writeEndTag("catalog-item");
			}

		} catch (Throwable ex) {
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("catalog");
			XMLWriter.writeEndTag("order");
		}
	}

	// No availability for any products
	private void order1(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order1 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "1"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[1];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605001"));
			partsOrderItem[0].setQuantity(new BigInteger("250"));
			partsOrderItem[0].setPrice((new BigDecimal(299.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[0]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}


	// Simple partial order
	private void order2(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order2 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "2"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[2];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605001"));
			partsOrderItem[0].setQuantity(new BigInteger("250"));
			partsOrderItem[0].setPrice((new BigDecimal(299.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[1] = new PartsOrderItem();
			partsOrderItem[1].setProductNumber(new BigInteger("605002"));
			partsOrderItem[1].setQuantity(new BigInteger("1"));
			partsOrderItem[1].setPrice((new BigDecimal(1499.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[1]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}

	// complex order with 2 product runs
	private void order3(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order3 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "3"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[3];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605003"));
			partsOrderItem[0].setQuantity(new BigInteger("8"));
			partsOrderItem[0].setPrice((new BigDecimal(5725.98)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[1] = new PartsOrderItem();
			partsOrderItem[1].setProductNumber(new BigInteger("605004"));
			partsOrderItem[1].setQuantity(new BigInteger("61"));
			partsOrderItem[1].setPrice((new BigDecimal(199.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[2] = new PartsOrderItem();
			partsOrderItem[2].setProductNumber(new BigInteger("605008"));
			partsOrderItem[2].setQuantity(new BigInteger("40"));
			partsOrderItem[2].setPrice((new BigDecimal(200.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[2]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}

	// Order rejected - InvalidProductCode
	private void order4(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order4 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "4"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[2];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605009"));
			partsOrderItem[0].setQuantity(new BigInteger("1"));
			partsOrderItem[0].setPrice((new BigDecimal(250.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[1] = new PartsOrderItem();
			partsOrderItem[1].setProductNumber(new BigInteger("605010"));
			partsOrderItem[1].setQuantity(new BigInteger("1"));
			partsOrderItem[1].setPrice((new BigDecimal(149.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);


		} catch (InvalidProductCodeType ex) {
			logResponse(null, demoUserId[3]);
		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}

	// no partial shipment for single product
	private void order5(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order5 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "5"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[1];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605001"));
			partsOrderItem[0].setQuantity(new BigInteger("20"));
			partsOrderItem[0].setPrice((new BigDecimal(299.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[4]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}

	// partial shipment with single brand order
	private void order6(ConfigurationType configHeader, String vendor) {
		logger.info("Placing order6 \"" + vendor + "\" ...");
		XMLWriter.writeStartTag("order", new AttributeNV("id", "6"));

		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[4];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605001"));
			partsOrderItem[0].setQuantity(new BigInteger("41"));
			partsOrderItem[0].setPrice((new BigDecimal(299.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[1] = new PartsOrderItem();
			partsOrderItem[1].setProductNumber(new BigInteger("605002"));
			partsOrderItem[1].setQuantity(new BigInteger("21"));
			partsOrderItem[1].setPrice((new BigDecimal(1499.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[2] = new PartsOrderItem();
			partsOrderItem[2].setProductNumber(new BigInteger("605004"));
			partsOrderItem[2].setQuantity(new BigInteger("46"));
			partsOrderItem[2].setPrice((new BigDecimal(199.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[3] = new PartsOrderItem();
			partsOrderItem[3].setProductNumber(new BigInteger("605007"));
			partsOrderItem[3].setQuantity(new BigInteger("41"));
			partsOrderItem[3].setPrice((new BigDecimal(100.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[5]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		} finally {
			XMLWriter.writeEndTag("order");
		}
	}

	// demo script
	private void demoScript(ConfigurationType configHeader) {
		logger.info("Invoking demoscript ...");
		try {
			PartsOrderType partsOrderType = new PartsOrderType();
			PartsOrderItem[] partsOrderItem = new PartsOrderItem[3];

			partsOrderItem[0] = new PartsOrderItem();
			partsOrderItem[0].setProductNumber(new BigInteger("605003"));
			partsOrderItem[0].setQuantity(new BigInteger("8"));
			partsOrderItem[0].setPrice((new BigDecimal(5725.98)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[1] = new PartsOrderItem();
			partsOrderItem[1].setProductNumber(new BigInteger("605004"));
			partsOrderItem[1].setQuantity(new BigInteger("61"));
			partsOrderItem[1].setPrice((new BigDecimal(199.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderItem[2] = new PartsOrderItem();
			partsOrderItem[2].setProductNumber(new BigInteger("605008"));
			partsOrderItem[2].setQuantity(new BigInteger("40"));
			partsOrderItem[2].setPrice((new BigDecimal(200.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

			partsOrderType.setItem(partsOrderItem);

			PartsOrderResponseType response = retailerStub.submitOrder(partsOrderType, getCustomerDetails(), configHeader);

			logResponse(response, demoUserId[0]);

		} catch (Throwable ex) {
			writeException(ex);
			ex.printStackTrace(System.out);
		}
	}

	private void writeException(Throwable ex) {
		XMLWriter.writeStartTag("error");

		XMLWriter.writeContent("error-message", ex.getMessage());
		StackTraceElement[] ste = ex.getStackTrace();
		for (int i=0; i<ste.length; i++) {
			XMLWriter.writeContent("error-message", ste[i].toString());
		}

		XMLWriter.writeEndTag("error");
	}
}

