/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.warehouse;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Iterator;
import java.util.ArrayList;

import java.math.BigInteger;

import javax.xml.rpc.Stub;
import com.sun.xml.rpc.client.StubBase;
import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.server.ServiceLifecycle;
import javax.xml.rpc.server.ServletEndpointContext;
import javax.xml.rpc.handler.MessageContext;

import com.sun.wsi.scm.util.WSIConstants;
import com.sun.wsi.scm.util.ConfigurationValidator;
import com.sun.wsi.scm.util.DBConnectionPool;
import com.sun.wsi.scm.util.URLFormatter;

import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;
import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationFaultType;

import com.sun.wsi.scm.manufacturer.cb.StartHeaderType;

import com.sun.wsi.scm.manufacturer.po.PurchOrdType;
import com.sun.wsi.scm.manufacturer.po.SubmitPOFaultType_Exception;

import com.sun.wsi.scm.manufacturer.ManufacturerService_Impl;
import com.sun.wsi.scm.manufacturer.ManufacturerPortType;

import com.sun.wsi.scm.warehouse.Item;
import com.sun.wsi.scm.warehouse.ItemList;
import com.sun.wsi.scm.warehouse.ItemShippingStatus;
import com.sun.wsi.scm.warehouse.ItemShippingStatusList;

import com.sun.wsi.scm.logging.LogEventRequestType;
import com.sun.wsi.scm.logging.LoggingFacilityService_Impl;
import com.sun.wsi.scm.logging.LoggingFacilityLogPortType;

import java.rmi.RemoteException;

import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;

public class WarehouseBPortTypeImpl implements WarehouseShipmentsPortType, ServiceLifecycle, WSIConstants {
	Hashtable dataHash = null;
	ConfigurationType configurationHeader = null;

	LoggingFacilityLogPortType logStub = null;
	LogEventRequestType logRequest = new LogEventRequestType();

	Log logger = null;

	DBConnectionPool dbPool = null;
	ServletEndpointContext servletEndpointContext = null;

	public void init(Object context) {
		logger = LogFactory.getLog(LOGGER);

		LoggingFacilityService_Impl logService = new LoggingFacilityService_Impl();
		logStub = logService.getLoggingFacilityPort();
		if (logger.isTraceEnabled())
			((StubBase)logStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		dataHash = new Hashtable();

		for (int i=0; i<WAREHOUSEB_DATA.length; i++) {
			Data data = new Data();
			data.setStock(WAREHOUSEB_DATA[i][0]);
			data.setMinimum(WAREHOUSEB_DATA[i][1]);
			data.setMaximum(WAREHOUSEB_DATA[i][2]);

			dataHash.put(new BigInteger(String.valueOf(PRODUCT_ID[i])), data);
		}

		servletEndpointContext = (ServletEndpointContext)context;
		dbPool = DBConnectionPool.getInstance(context);
	}


	public ItemShippingStatusList shipGoods(ItemList itemList, String customer, ConfigurationType configurationHeader) throws ConfigurationFaultType, RemoteException {
		if (logger.isTraceEnabled())
			logger.trace(WAREHOUSEB_SHIP_GOODS);
		this.configurationHeader = configurationHeader;
		ItemShippingStatus[] itemShippingStatusArray = new ItemShippingStatus[itemList.getItem().length];

		// ** - Validate the configurationHeader
		Hashtable roles = null;
		roles = ConfigurationValidator.validateHeader(configurationHeader);

		// Set the endpoint address of the logging facility
		// from the configurationHeader
		((Stub)logStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.LoggingFacility));

		String demoUserId = configurationHeader.getUserId();
		logRequest.setDemoUserID(demoUserId);
		logRequest.setServiceID(WAREHOUSEB_SHIP_GOODS);
		logRequest.setEventID("UC2-2-1");
		logRequest.setEventDescription("WarehouseB will determine its ability to ship " + StringConverter.getWarehouseProductNumbersAsString(itemList));
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());

		// ** - Set the status in the response. Replenish the stock if
		// the stock level falls below the minimum
		Vector stockRefillList = new Vector();

		// Segregate the shipped and unshipped items for logging
		ArrayList shippedItems = new ArrayList();
		ArrayList unshippedItems = new ArrayList();

		Item[] items = itemList.getItem();
		for (int i=0; i<items.length; i++) {
			if (!dataHash.containsKey(items[i].getProductNumber()))
				logger.error("Invalid product number: " + items[i].getProductNumber());
				//throw new JAXRPCException("Invalid product number: " + items[i].getProductNumber());

			Data data = (Data)dataHash.get(items[i].getProductNumber());

			itemShippingStatusArray[i] = new ItemShippingStatus();
			itemShippingStatusArray[i].setProductNumber(items[i].getProductNumber());

			// If requested quantity is more than the quantity in stock or
			// maximum quantity in stock, then the item cannot be shipped
			if ((items[i].getQuantity() > data.getMaximum()) ||
				(items[i].getQuantity() > data.getStock())) {
				if (logger.isTraceEnabled())
					logger.trace("Requested quantity is more than the quantity in stock or maximum quantity in stock and thus cannot be shipped");
				itemShippingStatusArray[i].setStatus(false);
				unshippedItems.add(items[i]);
				continue;
			} else {
				shippedItems.add(items[i]);
				itemShippingStatusArray[i].setStatus(true);
			}

			// Replenish stock if the inventory level has fallen 
			// below its minimum level
			if ((data.getStock() - items[i].getQuantity()) <= data.getMinimum()) {
				if (logger.isTraceEnabled())
					logger.trace("Replenishing stock since the inventory level has fallen below its minimum level");
				stockRefillList.add(items[i]);
			}
		}

		ItemList shippedItemList = new ItemList();
		Item[] shippedItemsArray = (Item[])shippedItems.toArray(new Item[0]);
		shippedItemList.setItem(shippedItemsArray);

		ItemList unshippedItemList = new ItemList();
		Item[] unshippedItemsArray = (Item[])unshippedItems.toArray(new Item[0]);
		unshippedItemList.setItem(unshippedItemsArray);

		logRequest.setEventID("UC2-2-2");
		logRequest.setEventDescription("WarehouseB is able to ship " + StringConverter.getWarehouseProductNumbersAsString(shippedItemList) + " and is unable to ship " + StringConverter.getWarehouseProductNumbersAsString(unshippedItemList));
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());

		try {
			replenishStock(stockRefillList, customer, roles, demoUserId);
		} catch (SubmitPOFaultType_Exception ex) {
			logger.error(ex);
		}
		ItemShippingStatusList itemShippingStatusList = new ItemShippingStatusList();
		itemShippingStatusList.setItemStatus(itemShippingStatusArray);

		return itemShippingStatusList;
	}

	/**
	* Batch process the stock replenishing from ManufacturerA,
	* ManufacturerB and ManufacturerC
	*/
	private void replenishStock(Vector stockRefillList, String customer, Hashtable roles, String demoUserId) throws SubmitPOFaultType_Exception, ConfigurationFaultType, RemoteException {
		if (logger.isTraceEnabled())
			logger.trace("Replenishing stock for " + WAREHOUSEB);

		ManufacturerService_Impl manufacturerService = new ManufacturerService_Impl();

		ManufacturerPortType manufacturerAStub = manufacturerService.getManufacturerAPort();
		((Stub)manufacturerAStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.ManufacturerA));
		if (logger.isTraceEnabled())
			((StubBase)manufacturerAStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		ManufacturerPortType manufacturerBStub = manufacturerService.getManufacturerBPort();
		((Stub)manufacturerBStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.ManufacturerB));
		if (logger.isTraceEnabled())
			((StubBase)manufacturerBStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		ManufacturerPortType manufacturerCStub = manufacturerService.getManufacturerCPort();
		((Stub)manufacturerCStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.ManufacturerC));
		if (logger.isTraceEnabled())
			((StubBase)manufacturerCStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		Vector mfrAItems = new Vector();
		Vector mfrBItems = new Vector();
		Vector mfrCItems = new Vector();

		// Classify items ordered per manufacturer in different buckets
		Iterator iter = stockRefillList.iterator();
		while (iter.hasNext()) {
			Item item = (Item)iter.next();
			int productNumber = item.getProductNumber().intValue();

			if ((productNumber == 605001) ||
				(productNumber == 605004) ||
				(productNumber == 605007))
				mfrAItems.add(item);
			else if ((productNumber == 605002) ||
					(productNumber == 6050005) ||
					(productNumber == 605008))
				mfrBItems.add(item);
			else if ((productNumber == 605003) ||
					(productNumber == 605006) ||
					(productNumber == 605009))
				mfrCItems.add(item);
			else
				logger.error("Invalid product number: \"" + String.valueOf(productNumber) + "\"");
		}

		refillFromManufacturer(manufacturerAStub, mfrAItems, customer, ConfigurationEndpointRole._ManufacturerAString, demoUserId);
		refillFromManufacturer(manufacturerBStub, mfrBItems, customer, ConfigurationEndpointRole._ManufacturerBString, demoUserId);
		refillFromManufacturer(manufacturerCStub, mfrCItems, customer, ConfigurationEndpointRole._ManufacturerCString, demoUserId);

	}

	void refillFromManufacturer(ManufacturerPortType manufacturerStub, Vector items, String customer, String role, String demoUserId) throws SubmitPOFaultType_Exception, ConfigurationFaultType, RemoteException {
		if (items.isEmpty())
			return;

		if (logger.isTraceEnabled())
			logger.trace("Replenishing stock for " + WAREHOUSEB + " from " + role);

		Iterator iter = items.iterator();

		ArrayList al = new ArrayList();

		// Convert Warehouse.Item to ManufacturerPO.Item
		while (iter.hasNext()) {
			Item item = (Item)iter.next();		// Warehouse.Item
			com.sun.wsi.scm.manufacturer.po.Item poItem = new com.sun.wsi.scm.manufacturer.po.Item();
			poItem.setID(item.getProductNumber());
			Data data = (Data)dataHash.get(item.getProductNumber());
			poItem.setQty(data.getMaximum() - (data.getStock() - item.getQuantity()));
			poItem.setPrice(PO_ITEM_PRICE);
			al.add(poItem);
		}

		com.sun.wsi.scm.manufacturer.po.Item[] itemArray = (com.sun.wsi.scm.manufacturer.po.Item[])al.toArray(new com.sun.wsi.scm.manufacturer.po.Item[0]);
		com.sun.wsi.scm.manufacturer.po.ItemList poItemList = new com.sun.wsi.scm.manufacturer.po.ItemList();
		poItemList.setItem(itemArray);

		PurchOrdType po = new PurchOrdType();

		// set to a constant number since this is not 
		// used for correlation anywhere
		po.setOrderNum(ORDER_NUMBER);
		po.setCustomerRef(customer);
		po.setItems(poItemList);
		// Set a constant price
		po.setTotal(PO_TOTAL_PRICE);		

		StartHeaderType startHeader = new StartHeaderType();
		String conversationID = demoUserId + "." + role;
		Vector setters = new Vector();
		setters.add(conversationID);
		setters.add(ConfigurationEndpointRole._WarehouseBString);
		setters.add(role);
		dbPool.insert(DBConnectionPool.CALLBACK, setters);

		startHeader.setConversationID(conversationID);
		MessageContext messageContext = servletEndpointContext.getMessageContext();
		HttpServletRequest request = (HttpServletRequest)messageContext.getProperty(com.sun.xml.rpc.server.http.MessageContextProperties.HTTP_SERVLET_REQUEST);
		startHeader.setCallbackLocation(URLFormatter.getCallbackURL(request));

		// Cast the manufacturer to the appropriate stub based upon the role
		boolean response;
		if (role.equals(ConfigurationEndpointRole._ManufacturerAString)) {
			response = manufacturerStub.submitPO(po, configurationHeader, startHeader);
		} else if (role.equals(ConfigurationEndpointRole._ManufacturerBString)) {
			response = manufacturerStub.submitPO(po, configurationHeader, startHeader);
		} else if (role.equals(ConfigurationEndpointRole._ManufacturerCString)) {
			response = manufacturerStub.submitPO(po, configurationHeader, startHeader);
		} else {
			logger.error("Invalid role: \"" + role + "\"");
		}
	}

	public void destroy() { }
}


